@@ -87,7 +87,9 @@ class Flow(object):
         https://console.developers.google.com/apis/credentials
     """
 
-    def __init__(self, oauth2session, client_type, client_config):
+    def __init__(
+            self, oauth2session, client_type, client_config,
+            redirect_uri=None):
         """
         Args:
             oauth2session (requests_oauthlib.OAuth2Session):
@@ -96,6 +98,9 @@ class Flow(object):
                 ``installed``.
             client_config (Mapping[str, Any]): The client
                 configuration in the Google `client secrets`_ format.
+            redirect_uri (str): The OAuth 2.0 redirect URI if known at flow
+                creation time. Otherwise, it will need to be set using
+                :attr:`redirect_uri`.
 
         .. _client secrets:
             https://developers.google.com/api-client-library/python/guide
@@ -107,6 +112,7 @@ class Flow(object):
         """Mapping[str, Any]: The OAuth 2.0 client configuration."""
         self.oauth2session = oauth2session
         """requests_oauthlib.OAuth2Session: The OAuth 2.0 session."""
+        self.redirect_uri = redirect_uri
 
     @classmethod
     def from_client_config(cls, client_config, scopes, **kwargs):
@@ -144,7 +150,8 @@ class Flow(object):
             google_auth_oauthlib.helpers.session_from_client_config(
                 client_config, scopes, **kwargs))
 
-        return cls(session, client_type, client_config)
+        redirect_uri = kwargs.get('redirect_uri', None)
+        return cls(session, client_type, client_config, redirect_uri)
 
     @classmethod
     def from_client_secrets_file(cls, client_secrets_file, scopes, **kwargs):
@@ -200,9 +207,9 @@ class Flow(object):
                 :class:`Flow` instance to obtain the token, you will need to
                 specify the ``state`` when constructing the :class:`Flow`.
         """
+        kwargs.setdefault('access_type', 'offline')
         url, state = self.oauth2session.authorization_url(
-            self.client_config['auth_uri'],
-            access_type='offline', **kwargs)
+            self.client_config['auth_uri'], **kwargs)
 
         return url, state
 
@@ -229,10 +236,9 @@ class Flow(object):
                 :meth:`credentials` to obtain a
                 :class:`~google.auth.credentials.Credentials` instance.
         """
+        kwargs.setdefault('client_secret', self.client_config['client_secret'])
         return self.oauth2session.fetch_token(
-            self.client_config['token_uri'],
-            client_secret=self.client_config['client_secret'],
-            **kwargs)
+            self.client_config['token_uri'], **kwargs)
 
     @property
     def credentials(self):
